import yaml

from charmhelpers import fetch
from charmhelpers.core import host, hookenv
from charmhelpers.core.templating import render
from charms.reactive import (
    when, when_not, set_state, remove_state
)
from charms.reactive.helpers import any_file_changed, data_changed


SVCNAME = 'prometheus-snmp-exporter'
PKGNAMES = ['prometheus-snmp-exporter']
PORT_DEF = 9116
SNMP_EXPORTER_YML = '/etc/prometheus/snmp.yaml'
SNMP_EXPORTER_YML_TMPL = 'snmp.yaml.j2'


def templates_changed(tmpl_list):
    return any_file_changed(['templates/{}'.format(x) for x in tmpl_list])


@when('snmp-exporter.do-install')
def install_packages():
    fetch.configure_sources()
    fetch.apt_update()
    fetch.apt_install(PKGNAMES)
    remove_state('snmp-exporter.do-install')


def validate_config():
    return yaml.safe_load(open(SNMP_EXPORTER_YML))


@when('alertmanager.do-reconfig-yaml')
def write_alertmanager_config_yaml():
    config = hookenv.config()
    render(source=SNMP_EXPORTER_YML_TMPL,
           target=SNMP_EXPORTER_YML,
           context={'config': config}
           )
    validate_config()
    hookenv.open_port(PORT_DEF)
    set_state('snmp-exporter.do-restart')
    remove_state('snmp-exporter.do-reconfig-yaml')


@when_not('snmp-exporter.started')
def setup_alertmanager():
    hookenv.status_set('maintenance', 'Installing software')
    install_packages()
    set_state('snmp-exporter.do-check-reconfig')


@when('snmp-exporter.started')
def check_config():
    set_state('snmp-exporter.do-check-reconfig')


@when('snmp-exporter.do-check-reconfig')
def check_reconfig_alertmanager():
    config = hookenv.config()
    install_opts = ('install_sources', 'install_keys')
    if any(config.changed(opt) for opt in install_opts):
        set_state('snmp-exporter.do-install')

    if data_changed('snmp-exporter.config', config):
        set_state('alertmanager.do-reconfig-yaml')

    if templates_changed([SNMP_EXPORTER_YML_TMPL]):
        set_state('snmp-exporter.do-reconfig-yaml')

    remove_state('snmp-exporter.do-check-reconfig')


@when('snmp-exporter.do-restart')
def restart_alertmanager():
    if not host.service_running(SVCNAME):
        hookenv.log('Starting {}...'.format(SVCNAME))
        host.service_start(SVCNAME)
    else:
        hookenv.log('Restarting {}, config file changed...'.format(SVCNAME))
        host.service_restart(SVCNAME)
    hookenv.status_set('active', 'Ready')
    set_state('snmp-exporter.started')
    remove_state('snmp-exporter.do-restart')


# Relations
@when('snmp-exporter.started')
@when('target.available')
def configure_snmp_exporter_relation(target):
    target.configure(PORT_DEF)
